-- |
-- Module      : System.Hourglass
-- License     : BSD-style
-- Maintainer  : Vincent Hanquez <vincent@snarc.org>
-- Stability   : experimental
-- Portability : unknown
--
-- Get the system timezone and current time value in multiple formats
--
-- This module will be depreciated in favor of Time.System
--
module System.Hourglass
    ( module Time.System
    ) where

import Time.System
